/* 
 * $Id: main.c,v 1.1.1.1 2000/08/14 18:46:16 labovit Exp $
 */

/***********************************************************************
Filename:	main.c
***********************************************************************/
#define _MAIN_C_

#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <mrt.h>
#include <trace.h>
#include <config_file.h>
#include "trace.h"
#include "hopList.h"
#include "traceloop.h"

#include "salamanderInterface.h"
#include "propertyList.h"

#include "tpd.h"
#include "Event.h"
#include "result.h"
#include "Globals.h"

#define SCHEDULE_NAME		"TPD Schedule"
#define CHECK_INTERVAL		5
#define DEFAULT_INTERVAL	30
#define TIMER_NAME		"TPD Timer"
#define LOGFILE_NAME		"/tmp/mrt_tpd.log"
#define CONFIGFILE_NAME		"./tpd.cfg"
#define THREAD_NAME		"Traceroute Thread"
#define SALY_THREAD_NAME	"Salamander Thread"
//#define DEFAULT_SALAMANDER	"salamander.merit.edu"
#define DEFAULT_SALAMANDER	"198.108.0.13"
#define DEFAULT_UII_PORT	"5665"

#ifndef UII_CONFIG
#define UII_CONFIG		2
#endif /* UII_CONFIG */

const char	COMMAND_CONFIG_SALSERVER[]	= {"salserver"};
const char	MYHEADER[] = {
	"################################################################"};

const char	USAGE[] = {
	"Usage: %s [-f config_file] [-tmax max_trace_filesize] "
		"[-p uii_port ] [-k UII key] [-s salamander_server] "
		"[-v] [-h]\n"};

									
#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/*int gethostname(char *name, int namelen);*/

int main(int argc, char *argv[]);
void InitTPD(void);
void LoadTPDConfiguration(char *filename);
void InitUserInterface(char *Password);
void CheckForReadyEvents(void);
void *StartNewTraceRoute(void *);
void *SalamanderThread(void *);
void MainLoop(void);
static void config_create_default(void);
int config_salserver(uii_connection_t *uii, char *Server);

int
main(int argc, char *argv[]) {
	char		*name = (char *) argv[0];
	extern char	*optarg;		// getopt stuff
	extern int	optind;			// getopt stuff
	char		*config_file	= CONFIGFILE_NAME,
			*UIIKey		= NULL;

	int		errors = 0;
	char		c;

	if (seteuid(geteuid()) || setuid(getuid())) {
		fprintf (stderr,
			"tpd: must be run with root priviledges\n");
		exit (0xdeadbeef);
	}

	TPDGlobals.TracePtr = New_Trace();
	TPDGlobals.SalamanderServerPtr	= DEFAULT_SALAMANDER;
	TPDGlobals.UIIPort		= DEFAULT_UII_PORT;
	init_mrt(TPDGlobals.TracePtr);
	init_uii(TPDGlobals.TracePtr);
	set_trace(TPDGlobals.TracePtr,
		TRACE_FLAGS, TR_ERROR | NORM,
		TRACE_LOGFILE, LOGFILE_NAME, NULL);

	while ((c = getopt(argc, argv, "vf:p:k:s:")) != -1) {
		switch (c) {
			/*  verbose */
			case 'v':
				set_trace(TPDGlobals.TracePtr,
					TRACE_FLAGS, TR_ERROR | NORM,
					TRACE_LOGFILE, "stdout", NULL);
				break;
			case 'f':
				printf("Config file is %s\n", optarg);
				fflush(stdout);
				config_file = optarg;
				break;
			case 'p':
				TPDGlobals.UIIPort = optarg;
				break;
			case 'k':
				UIIKey = optarg;
				break;
			case 's':
				TPDGlobals.SalamanderServerPtr = optarg;
				break;
			default:
				errors++;
		}
	}

	if (errors) {
		fprintf(stderr, USAGE, name);
		printf ("\nTPD %s compiled on %s\n\n",
		MRT_VERSION, __DATE__);
		exit (1);
	}
 
	InitTPD();
	InitUserInterface(UIIKey);
	LoadTPDConfiguration(config_file);

	listen_uii2(TPDGlobals.UIIPort); 

	// Initialize all the mutex locks that we need.
	// I guess if we don't have PThreads, we're screwed...
	pthread_mutex_init(&TPDGlobals.EventListLock, NULL); 
	pthread_mutex_init(&TPDGlobals.ResultListLock, NULL); 
  
	TPDGlobals.Running = TRUE;
	TPDGlobals.NumRunningThreads = 0;
	mrt_thread_create(SALY_THREAD_NAME, NULL, SalamanderThread,
		(void *)NULL);
	MainLoop(); 
	return 0; 
}

/***********************************************************************
Initialize the Traceroute Probe Daemon.
***********************************************************************/
void InitTPD() {  
#ifdef HAVE_LIBPTHREAD 
	sigset_t set; 
#endif 
	char	*ChannelNamePtr;

	ChannelNamePtr = TPDGlobals.ChannelName;
	memset(ChannelNamePtr, 0, MAXHOSTNAMELEN);

	ChannelNamePtr[0] = 'T';
	ChannelNamePtr[1] = 'P';
	ChannelNamePtr[2] = 'D';
	ChannelNamePtr[3] = ':';

	ChannelNamePtr += 4;

	gethostname(TPDGlobals.HostName, sizeof(TPDGlobals.HostName));

	memcpy(ChannelNamePtr, &TPDGlobals.HostName, MAXHOSTNAMELEN - 3);
 
	TPDGlobals.SalamanderServerPrefixPtr = string_toprefix( 
		TPDGlobals.SalamanderServerPtr, 
		TPDGlobals.TracePtr); 

	if (!TPDGlobals.SalamanderServerPrefixPtr) { 
		trace (ERROR, TPDGlobals.TracePtr, "Server not found\n"); 
		exit(0xdeadbeef); 
	} 

	TPDGlobals.SalamanderKey = 123456; 
	memset(&TPDGlobals.EventListHead, 0, sizeof(TPDEvent)); 
	memset(&TPDGlobals.ResultListHead, 0, sizeof(SalyData_t));
	TPDGlobals.ResultListTail = &TPDGlobals.ResultListHead;

	TPDGlobals.NumQueuedResults = 0;

	// Mask off some signals 
#ifdef HAVE_LIBPTHREAD 
	sigemptyset(&set); 
	sigaddset(&set, SIGALRM); 
	sigaddset(&set, SIGHUP); 
	pthread_sigmask(SIG_BLOCK, &set, NULL); 
#endif 
} 

/***********************************************************************
Load Config stuff
***********************************************************************/ 
void LoadTPDConfiguration(char *config_file) {

	int	ret;

	// read configuration here
	ret = config_from_file (TPDGlobals.TracePtr, config_file);
	if (ret < 0) {
		// No configuration file, so default to no password
	        UII->initial_state = 1;
		config_create_default();
	}
	else {
		// Configuration file found, start with password
		// prompt
		UII->initial_state = 0;
	}

}

/*********************************************************************** 
Initialize all components of the user interface system. 
***********************************************************************/ 
void
InitUserInterface(char *Password) { 

	set_uii(UII, UII_PROMPT, 0, "Password> "); 
	set_uii(UII, UII_PROMPT, 1, "TPD> "); 
	set_uii(UII, UII_PROMPT, UII_CONFIG, "Config> "); 

        uii_add_command2(UII_UNPREV, COMMAND_MATCH_FIRST,
		"", uii_check_passwd, "password");

	// Main menu options
	uii_add_command2 (UII_NORMAL, COMMAND_NORM,
		"config", 
		(void *) start_config, 
		"Configure TPD options");
	uii_add_command2 (UII_NORMAL, COMMAND_NORM,
		"show config",
		show_config,
                "Display current configuration");

	uii_add_command2 (UII_NORMAL, COMMAND_NORM,
		"write",
		(void *) config_write, 
		"Save configuration to disk");

	uii_add_command2(UII_NORMAL, COMMAND_NORM, 
		"show traceroutes", 
		ShowTraceroutes, 
		"List all registered Traceroute events"); 

	// Config menu options
	uii_add_command2(UII_CONFIG, COMMAND_NORM, 
		"traceroutenorm %s %d",
		AddTraceroute,
		"Add a TPD traceroute host");

	uii_add_command2(UII_CONFIG, COMMAND_NORM, 
		"traceroutefull %s %d %d %d",
		AddTracerouteFull,
		"Add a TPD traceroute host with all parameters");

	uii_add_command2(UII_CONFIG, COMMAND_NORM, 
		"changeinterval %d %d",
		ModifyTracerouteInterval,
		"Modify size of traceroute packets for TPD host");

	uii_add_command2(UII_CONFIG, COMMAND_NORM, 
		"changesize %d %d",
		ModifyTracerouteSize,
		"Modify size of traceroute packets for TPD host");

	uii_add_command2(UII_CONFIG, COMMAND_NORM,
		"changettl %d %d",
		ModifyTracerouteTTL,
		"Modify TTL of traceroute packets for TPD host");

	uii_add_command2(UII_CONFIG, COMMAND_NORM, 
		"remove %d",
		RemoveTraceroute,
		"Remove a TPD traceroute host"); 

	uii_add_command2(UII_CONFIG, COMMAND_NORM, 
		"show traceroutes", 
		ShowTraceroutes, 
		"List all registered Traceroute events"); 

	uii_add_command2 (UII_CONFIG, COMMAND_NORM,
		"salserver",
		config_salserver,
		"Configure Salamander Server options");

}


/*********************************************************************** 

***********************************************************************/
static void
config_create_default(void) {

	char *tmp;

	tmp = strdup (MYHEADER);
	config_add_module (0, "comment", get_comment_config, tmp);

	tmp = malloc (512);
	sprintf (tmp, "# TPD -- MRT version %s ", MRT_VERSION);
	config_add_module (0, "comment", get_comment_config, tmp);
		
	tmp = strdup(MYHEADER);
	config_add_module (0, "comment", get_comment_config, tmp);

	config_add_module (0, "comment", get_comment_config, strdup ("#"));
	config_add_module (0, "debug", get_debug_config, NULL);
	config_add_module (0, "comment", get_comment_config, strdup ("#"));  
}


/*********************************************************************** 
Choose the proper Salamander Server.
***********************************************************************/
int
config_salserver (uii_connection_t *uii, char *Server) {


	TPDGlobals.SalamanderServerPtr = Server;
	config_notice(NORM, uii, "%s\r\n");

	TPDGlobals.SalamanderServerPrefixPtr =
		string_toprefix(
			TPDGlobals.SalamanderServerPtr,
			TPDGlobals.TracePtr);

	if (!TPDGlobals.SalamanderServerPrefixPtr) {
		config_notice(NORM,
			uii,
			"Failed to obtain a prefix for %s.  Aborting\r\n",
			Server);
		free(Server);
		return -1;
	}

	return 1;
}


/***********************************************************************
Check all pending events and spawn off the ones that need to be run.

NOTE: This is super inefficient, since I haven't gotten around to
implementing a heap.
***********************************************************************/
void
CheckForReadyEvents() {

	TPDEvent	*Event;
	struct timeval	CurrentTime;
	mrt_thread_t	*TID;

	gettimeofday(&CurrentTime, NULL);

	if (!TPDGlobals.Running)
		return;

	// Check to see if the next event is ready.  If so, then we need
	// to start a new thread and begin a traceroute.
	// Stay in a loop so we start every single ready event.
	GetEventListLock();
	do {
		Event = GetNextEvent();
		if (Event &&
			(CurrentTime.tv_sec > Event->NextStartTime.tv_sec)) {
			trace (TRACE, TPDGlobals.TracePtr,
				"Event was ready: starting traceroute\n");

			RemoveEvent(Event);
			Event->NextStartTime.tv_sec =
				CurrentTime.tv_sec +
				Event->Interval.tv_sec;
			Event->NextStartTime.tv_usec =
				CurrentTime.tv_usec +
				Event->Interval.tv_usec;
			AddEvent(Event);
			if (Event->Running >= 0)
				Event->Running++;
			else
				Event->Running--;

			TPDGlobals.NumRunningThreads++;
			trace(NORM, TPDGlobals.TracePtr,
				"Adding Traceroute Thread:"
				"%ld active\n",
				TPDGlobals.NumRunningThreads);

			TID = mrt_thread_create(THREAD_NAME, NULL,
				StartNewTraceRoute, (void *)Event);
			if (!TID) {
				trace(ERROR, TPDGlobals.TracePtr,
					"Unable to start thread\n");
				if (Event->Running > 0)
					Event->Running--;
				else {
					Event->Running++;
					if (!Event->Running)
						DestroyEvent(Event);
				}
			}
		}
		else
			break;
	} while (1);
	ReleaseEventListLock();
}

/***********************************************************************
Initiate a traceroute for this particular event.

NOTE: This should be started in its own thread.
***********************************************************************/
void *
StartNewTraceRoute(void *args) {

	TPDEvent	*Event = (TPDEvent *)args;
	HopListNode	*hopList;
	prefix_t	*NewPrefix;
	int		verbose = 1;

#ifdef HAVE_LIBPTHREAD
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, SIGALRM);
	sigaddset(&set, SIGHUP);
	pthread_sigmask(SIG_BLOCK, &set, NULL);
#endif

	// Start a traceroute here
	trace(NORM, TPDGlobals.TracePtr, "Starting Thread\n");
	NewPrefix = string_toprefix(Event->DstAddr, NULL);
	if (!NewPrefix) {
		trace (NORM, TPDGlobals.TracePtr,
			"Unable to get prefix for %s\n",
			Event->DstAddr);
		GetEventListLock();
		// Fix the list of pending events
		if (Event->Running > 0)
			Event->Running--;
		else {
			Event->Running++;
			if (!Event->Running)
				DestroyEvent(Event);
		}
		trace(NORM, TPDGlobals.TracePtr,
			"Deleted Traceroute Thread: %ld left\n",
			--TPDGlobals.NumRunningThreads);
		ReleaseEventListLock();
		trace(NORM, TPDGlobals.TracePtr, "Ending Thread\n");
		mrt_thread_exit();
		return NULL;
	}
	else {
		trace (NORM, TPDGlobals.TracePtr,
			"Got prefix for %s\n",
			Event->DstAddr);
	}

	trace (NORM, TPDGlobals.TracePtr, "Sending Traceroute to %s\n",
		Event->DstAddr);
	hopList = SendTraceRoute(
		NewPrefix,
		Event->TTL,
		Event->PacketSize,
		verbose);
	trace (NORM, TPDGlobals.TracePtr, "Traceroute Sent to %s\n",
		Event->DstAddr);

	// Queue the results for delivery to the Salamander Server
	// at a later time
	if (!hopList) {
		trace(NORM, TPDGlobals.TracePtr, "Traceroute failed?\n");
	}
	else {
		CreateResultPacket(Event, hopList);

		trace (NORM, TPDGlobals.TracePtr,
			"Queued results for Salamander\n");

		FreeHopList(hopList);
	}

	Delete_Prefix(NewPrefix);

	GetEventListLock();
	// Fix the list of pending events
	if (Event->Running >= 0)
		Event->Running--;
	else {
		Event->Running++;
		if (!Event->Running)
			DestroyEvent(Event);
	}
	trace(NORM, TPDGlobals.TracePtr,
		"Deleted Traceroute Thread: %ld left\n",
		--TPDGlobals.NumRunningThreads);
	ReleaseEventListLock();

	trace(NORM, TPDGlobals.TracePtr, "Ending Thread\n");
	mrt_thread_exit();
	return NULL;
}

/***********************************************************************
This is the main loop of the program.
***********************************************************************/
void *
SalamanderThread(void *args) {
#ifdef HAVE_LIBPTHREAD
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, SIGALRM);
	sigaddset(&set, SIGHUP);
	pthread_sigmask(SIG_BLOCK, &set, NULL);
#endif

	trace(NORM, TPDGlobals.TracePtr,
		"Starting Salamander Thread\n");

	ReportResults();
	mrt_thread_exit();
	return NULL;
}


/***********************************************************************
This is the main loop of the program.
***********************************************************************/
void
MainLoop() {
	do {
		sleep(CHECK_INTERVAL);
		CheckForReadyEvents();
	} while (TPDGlobals.Running);
}
