#define _RESULT_C_

/*
#include <netinet/in.h>
*/

#include <mrt.h>
#include "hopList.h"
#include "salamanderInterface.h"
#include "Globals.h"
#include "result.h"

#define MAX_QUEUED_RESULTS	512

char	KEY_COMMAND[]		= {COMMAND_PROPERTY};
char	KEY_NAME[]		= {NAME_PROPERTY};
char	KEY_TIME[]		= {"TIME"};

char	COMMAND_PUBLISH[]	= {PUBLISH_COMMAND};

/***********************************************************************
This function loops as it attempts to blindly send all waiting data to
the salamander server.  If is smart enough to detect whether or not
data is available, if it is connected to the server and/or
if it needs to reconnect.
***********************************************************************/
void
ReportResults(void) {

	SalyData_t	*SalyDataPtr;
	int		ret;
	char		*SalyString;

	do {
		if (!TPDGlobals.SalamanderServerPtr) {
			trace (NORM,  TPDGlobals.TracePtr,
				"No Salamander Server\n");
			sleep(300);
			continue;
		}
		else if (!salamander_connected()) {
			SalyString = prefix_toa(
				TPDGlobals.SalamanderServerPrefixPtr);
			trace (NORM,  TPDGlobals.TracePtr,
				"Connecting to Salamander @ %s:%lu\n",
				SalyString,
				TPDGlobals.SalamanderKey);
			free(SalyString);
			ret = connectToSalamanderServer(
				TPDGlobals.SalamanderServerPrefixPtr,
				TPDGlobals.SalamanderKey);
			if (ret != SALAMANDER_OK) {
				// Connection to salamander has failed
				trace(NORM, TPDGlobals.TracePtr,
					"Connect to Salamander failed: %d\n",
					ret);
			}
			else {
				trace (NORM, TPDGlobals.TracePtr,
					"Connected to Salamander\n");
			}
			
		}
		else {
			trace(NORM, TPDGlobals.TracePtr, "WTF?!?\n");
		}
		while (salamander_connected()) {
			GetResultListLock();
			SalyDataPtr = GetResultListHead();
			ReleaseResultListLock();
			if (SalyDataPtr) {
				ret = salamanderSendServerData(
					SalyDataPtr->PList,
					SalyDataPtr->DataPtr,
					SalyDataPtr->Size);
				if (ret != SALAMANDER_OK) {
					// some error occurred
					trace(NORM | ERROR,
						TPDGlobals.TracePtr,
						"Send failed\n");
					disconnectFromSalamanderServer();
					break;
				}
				else {
					// success, delete result
					trace(NORM, TPDGlobals.TracePtr,
						"Send succeeded\n");
					GetResultListLock();
					RemoveResultListHead(SalyDataPtr);
					destroyPropertyList(
						SalyDataPtr->PList);
					free(SalyDataPtr->DataPtr);
					free(SalyDataPtr);
					ReleaseResultListLock();
				}
			}
			else
				sleep(60);
		}
		sleep(180);
	} while (1);
}

/***********************************************************************
Given a TPDEvent and its corresponding result packet, this will
create a packet to send to salamander and queue it up for delivery.
***********************************************************************/
void
CreateResultPacket(TPDEvent *Event, HopListNode *HopList) {

	SalyData_t	*SalyDataPtr;

	SalyDataPtr = (SalyData_t *)calloc(1, sizeof(SalyData_t));

	if (!HopList)
		return;

	SalyDataPtr->PList = CreateTPD_PList(Event);
	if (!SalyDataPtr->PList) {
		trace(ERROR, TPDGlobals.TracePtr,
			"PList wasn't created\n");
		return;
	}

	trace(NORM, TPDGlobals.TracePtr,
		"Creating Salamander Data\n");

	SalyDataPtr->DataPtr = BundleData(HopList);
	SalyDataPtr->Size = CalcResultPacketSize(HopList);

	GetResultListLock();
	AppendToResultList(SalyDataPtr);

	ReleaseResultListLock();
}

/***********************************************************************
This determines the size of the packet necessary for storing a partcular
HopList.
***********************************************************************/
int
CalcResultPacketSize(HopListNode *HopList) {

	HopListNode	*HopListPtr;
	int		count = 0;

	HopListPtr = HopList;
	while (HopListPtr) {
		HopListPtr = HopListPtr->Next;
		count++;
	}
	return (count * sizeof(ResultPacket));
}

/***********************************************************************
Given a TPDEvent, this function will create the corresponding PList.
***********************************************************************/
plist_t
CreateTPD_PList(TPDEvent *EventPtr) {

	plist_t		PList;
	struct timeval	CurrentTime;
	char		Time[TIME_STRING_SIZE];
	char		ChannelName[MAXHOSTNAMELEN];
	char		*ChannelNamePtr;
	int		ChannelNameLen;
	int		DstAddrLen;
	int		ret;

	// Create proper items for time field
	gettimeofday(&CurrentTime, NULL);
	sprintf(Time, "%lu", CurrentTime.tv_sec);

	// Create proper channel name
	ChannelNameLen = strlen(TPDGlobals.ChannelName);
	DstAddrLen = strlen(EventPtr->DstAddr);
	strncpy(ChannelName, TPDGlobals.ChannelName, ChannelNameLen);
	ChannelName[ChannelNameLen++] = ':';
	ChannelNamePtr = ChannelName + ChannelNameLen;
	if (DstAddrLen + ChannelNameLen > MAXHOSTNAMELEN) {
		strncpy(ChannelNamePtr, EventPtr->DstAddr,
			MAXHOSTNAMELEN - ChannelNameLen);
		ChannelName[MAXHOSTNAMELEN - 1] = '\0';
		trace(NORM, TPDGlobals.TracePtr,
			"1)Created PList for %s\n", ChannelName);
	}
	else {
		strncpy(ChannelNamePtr, EventPtr->DstAddr,
			DstAddrLen);
		ChannelName[DstAddrLen + ChannelNameLen] = '\0';
		trace(NORM, TPDGlobals.TracePtr,
			"2)Created PList for %s\n", ChannelName);
	}


	PList = createPropertyList();
	if (!PList)
		return PList;
	ret = updateProperty(PList, KEY_NAME, ChannelName);
	if (!ret)
		return PList;
	ret = updateProperty(PList, KEY_TIME, Time);
	if (!ret)
		return PList;
	ret = updateProperty(PList, KEY_COMMAND, COMMAND_PUBLISH);
	if (!ret)
		return PList;

	return PList;	
}

/***********************************************************************
This will take a Hoplist and convert it into a byte array for delivery
to salamander.
***********************************************************************/
char *
BundleData(HopListNode *HopList) {

	HopListNode	*HopListPtr;
	ResultPacket	*resultPtr;
	char		*dataBuffer;
	int		size = 0;
	struct sockaddr_in	*Addr;

	size = CalcResultPacketSize(HopList);
	if (!size)
		return NULL;

	dataBuffer = (char *)calloc(1, size);

	HopListPtr = HopList;

	resultPtr = (ResultPacket *)dataBuffer;
	while (HopListPtr) {
		// Address is already in network byte order
		Addr = (struct sockaddr_in *)&HopListPtr->Addr;
		sprintf(resultPtr->Address,
			"%03u.%03u.%03u.%03u",
			(uint)(Addr->sin_addr.s_addr >> 24) & 0xff,
			(uint)(Addr->sin_addr.s_addr >> 16) & 0xff,
			(uint)(Addr->sin_addr.s_addr >> 8) & 0xff,
			(uint)(Addr->sin_addr.s_addr) & 0xff);

		resultPtr->Address[15] = '\0';
/*
		memcpy(&resultPtr->Address,
				&(Addr->sin_addr),
				sizeof(struct in_addr));
*/
		resultPtr->TimeUSec = htons(
				(int)HopListPtr->RoundTripTime * 1000000);
		resultPtr++;
		HopListPtr = HopListPtr->Next;
	}

	return dataBuffer;
}


/***********************************************************************
Get a pointer to the head of the Global results list.
***********************************************************************/
SalyData_t *
GetResultListHead(void) {
	return TPDGlobals.ResultListHead.Next;
}

/***********************************************************************
Remove the head of the result list from the queue.

NOTE: This must only be used after result mutex has been obtained...
***********************************************************************/
void
RemoveResultListHead(SalyData_t *SalyDataPtr) {
	if (SalyDataPtr == TPDGlobals.ResultListTail)
		TPDGlobals.ResultListTail = &TPDGlobals.ResultListHead;
	TPDGlobals.ResultListHead.Next =
		TPDGlobals.ResultListHead.Next->Next;
	TPDGlobals.NumQueuedResults--;
}

/***********************************************************************
Add the new Salamander packet to the queue of results.

NOTE: This must only be used after result mutex has been obtained...
***********************************************************************/
void
AppendToResultList(SalyData_t *SalyDataPtr) {

	if (TPDGlobals.NumQueuedResults == MAX_QUEUED_RESULTS) {
		trace (NORM,  TPDGlobals.TracePtr,
			"Dropping result packets\n");
	}
	else {
		TPDGlobals.ResultListTail->Next = SalyDataPtr;
		TPDGlobals.ResultListTail = SalyDataPtr;
		TPDGlobals.NumQueuedResults++;
	}
}
